from .auxfunc import (discretization, sjoin, condentropy)

from .sentropies import *
from .jentropies import *
from .ternary import *
from .coordsentropic import entcoords
from .entropytriangle import entriangle, entriangle_list